
BASIC_FORM_CSS = """\
div.qpform {
    float: left;
}

div.qpform div.widget,
div.qpform div.submit {
    clear: left;
    padding: 0.5ex;
    margin: 0.5ex;
}

div.qpform div.widget div.widget,
div.qpform div.submit div.widget {
    margin: 0;
    padding: 0;
    clear: none;
}

div.qpform div.title {
    font-weight: bold;
}

div.qpform div.close {
    clear: left;
}

div.qpform div.submit div.widget {
    clear: none;
    float: left;
    margin-right: 0.5ex;
}

div.qpform div.error {
    color: #c30;
    font-size: small;
    margin-top: 0.2ex;
}

div.qpform div.hint {
    font-size: small;
    font-style: italic;
    margin-top: 1ex;
    width: 40ex;
}

div.qpform div.errornotice {
    color: #c30;
    padding: 1ex;
    margin: 1ex;
    width: 40ex;
}
"""
