"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qp/demo/hello/slash.py $
$Id: slash.py 28044 2006-03-09 16:35:08Z dbinger $
"""
from qp.pub.publish import Publisher

class SitePublisher (Publisher):

    configuration = dict(
        http_address=('', 8000),
        as_https_address=('localhost', 9000),
        https_address=('localhost', 10000),
        )

    def process_hit(self, hit):
        hit.get_response().set_body('Hello')

