"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qp/fill/__init__.py $
$Id: __init__.py 26983 2005-06-29 18:57:59Z dbinger $
"""
from qpy.compile import compile_qpy_files
compile_qpy_files(__path__[0])
